//
//  TADCustomBanner.swift
//  AdmobSDKTest
//
//  Created by Mira on 2021/6/16.
//

import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomBanner: NSObject, GADCustomEventBanner, TADBannerViewDelegate {
    weak var delegate: GADCustomEventBannerDelegate?
    var bannerView: TADBannerView?

    override required init() {
        super.init()
    }

    func requestAd(_ adSize: GADAdSize, parameter serverParameter: String?, label serverLabel: String?, request: GADCustomEventRequest) {
        let tadRequest = TADRequest()
        tadRequest.isTesting = request.isTesting
        tadRequest.setLocationWithLatitude(request.userLatitude, longitude: request.userLongitude, accuracy: request.userLocationAccuracyInMeters)

        let adRatio: CGFloat = floor((adSize.size.width/adSize.size.height)*100)/100
        let ta1200x627AdRatio: CGFloat = floor((1200.0/627)*100)/100

        if GADAdSizeEqualToSize(adSize, GADAdSizeBanner) {
            self.bannerView = TADBannerView(adSize: kTADAdSizeBanner)
        } else if GADAdSizeEqualToSize(adSize, GADAdSizeMediumRectangle) {
            self.bannerView = TADBannerView(adSize: kTADAdSizeMediumRectangle)
        } else if adRatio == ta1200x627AdRatio {
            self.bannerView = TADBannerView(adSize: TADAdSize1200x627RatioBannerWithWidth(adSize.size.width))
        } else {
            let error = NSError(domain: "com.TAmedia.TADCustomBanner", code: GADErrorCode.mediationInvalidAdSize.rawValue, userInfo: nil)
            self.delegate?.customEventBanner(self, didFailAd: error)
        }

        self.bannerView?.delegate = self
        self.bannerView?.adUnitID = serverParameter
        self.bannerView?.turnOnMicrophone = false
        self.bannerView?.rootViewController = self.delegate?.viewControllerForPresentingModalView
        self.bannerView?.load(tadRequest)
    }
}

// MARK: TADBannerViewDelegate

extension TADCustomBanner {
    func adViewDidReceiveAd(_ view: TADBannerView!) {
        print("CustomBanner - adViewDidReceiveAd!!")
        self.delegate?.customEventBanner(self, didReceiveAd: view)
    }

    func adView(_ view: TADBannerView!, didFailToReceiveAdWithError error: TADRequestError!) {
        print("CustomBanner - didFailToReceiveAdWithError!! error: \(String(describing: error))")
        self.delegate?.customEventBanner(self, didFailAd: error)
    }

    func adViewWillPresentScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillPresentScreen!!")
        self.delegate?.customEventBannerWillPresentModal(self)
    }

    func adViewWillDismissScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillDismissScreen!!")
        self.delegate?.customEventBannerWillDismissModal(self)
    }

    func adViewDidDismissScreen(_ adView: TADBannerView!) {
        print("CustomBanner - adViewDidDismissScreen!!")
        self.delegate?.customEventBannerDidDismissModal(self)
    }

    func adViewWillLeaveApplication(_ adView: TADBannerView!) {
        print("CustomBanner - adViewWillLeaveApplication!!")
        self.delegate?.customEventBannerWillLeaveApplication(self)
    }
}
