//
//  TADCustomInterstitial.swift
//  AdmobSDKTest
//
//  Created by Mira on 2021/6/17.
//

import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomInterstitial: NSObject, GADCustomEventInterstitial, TADInterstitialDelegate {
    weak var delegate: GADCustomEventInterstitialDelegate?
    var interstitialView: TADInterstitial?

    override required init() {
        super.init()
    }

    func requestAd(withParameter serverParameter: String?, label serverLabel: String?, request: GADCustomEventRequest) {
        // 將 GADCustomEventRequest 物件裡的資訊轉入 TADRequest 物件
        let tadRequest = TADRequest()
        tadRequest.isTesting = request.isTesting
        tadRequest.location = request.userLocationDescription
        tadRequest.setLocationWithLatitude(request.userLatitude, longitude: request.userLongitude, accuracy: request.userLocationAccuracyInMeters)

        self.interstitialView = TADInterstitial()
        self.interstitialView?.delegate = self
        self.interstitialView?.adUnitID = serverParameter
        self.interstitialView?.turnOnMicrophone = false
        self.interstitialView?.load(tadRequest)
    }

    func present(fromRootViewController rootViewController: UIViewController) {
        self.interstitialView?.present(fromRootViewController: rootViewController)
    }
}

// MARK: TADInterstitialDelegate

extension TADCustomInterstitial {
    func interstitialDidReceiveAd(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialDidReceiveAd!!")
        self.delegate?.customEventInterstitialDidReceiveAd(self)
    }

    func interstitial(_ ad: TADInterstitial!, didFailToReceiveAdWithError error: TADRequestError!) {
        print("CustomInterstitial - didFailToReceiveAdWithError!! error: \(String(describing: error))")
        self.delegate?.customEventInterstitial(self, didFailAd: error)

        // 需手動設為 nil，auto release 在 admob SDK 會造成在未在 main thread 執行
        self.interstitialView?.delegate = nil
        self.interstitialView = nil
    }

    func interstitialWillPresentScreen(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillPresentScreen!!")
        self.delegate?.customEventInterstitialWillPresent(self)
    }

    func interstitialDidDismissScreen(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillDismissScreen!!")
        self.delegate?.customEventInterstitialWillDismiss(self)
    }

    func interstitialWillLeaveApplication(_ ad: TADInterstitial!) {
        print("CustomInterstitial - interstitialWillLeaveApplication!!")
        self.delegate?.customEventInterstitialWillLeaveApplication(self)
    }
}
