//
//  TADCustomNative.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2022/1/17.
//

import Foundation
import GoogleMobileAds
import TAMediaAdsFramework

class TADCustomNative: NSObject, GADCustomEventNativeAd {
    weak var delegate: GADCustomEventNativeAdDelegate?
    private var nativeAd: TADNativeAd?
    
    override required init() {
        super.init()
    }
    
    deinit {
        print("TADCustomNative deinit")
    }
    
    func request(withParameter serverParameter: String, request: GADCustomEventRequest, adTypes: [Any], options: [Any], rootViewController: UIViewController) {
        loadTADNativeAd(adUnitId: serverParameter)
    }
    
    func handlesUserClicks() -> Bool {
        return true
    }
    
    func handlesUserImpressions() -> Bool {
        return true
    }
}

extension TADCustomNative {
    private func loadTADNativeAd(adUnitId: String) {
        let request = TADRequest()
        request.showLog = true
        let option = TADNativeAdOptions()
        /// 參見 http://wiki.tamedia.com.tw/iosDoc/Classes/TADNativeAdOptions.html
//        option.disableImageLoading = true //SDK 將不自動下載圖片 TADMediatedUnifiedNativeAd 內需更改
//        option.mediaPreferImage = false //只會呈現圖片廣告
//        option.startUnmuted = true     //影片廣告會以非靜音方式呈現
//        option.customControlsRequested = true //
//        option.allowAudioSessionControl = false //設為false則需自行設定影音廣告出現時 AVAudioSession 狀態
        nativeAd = TADNativeAd(adUnitId: adUnitId)
        nativeAd?.delegate = self
        nativeAd?.load(request, withOption: option)
    }
}

extension TADCustomNative: TADNativeAdDelegate {
    func nativeAdDidReceive(_ ad: TADNativeAd!) {
        let nativeAdView = TADNativeAdView()
        let mediaView = TADNativeMediaView(frame: .zero)
        mediaView.mediaContent = ad.adContent.mediaContent
        /// 參見 http://wiki.tamedia.com.tw/iosDoc/Classes/TADNativeMediaView.html
//        mediaView.setCallToActionVisible(false) //是否顯示 CTA 按鈕
//        mediaView.setCallToActionTextSize(12) //設定 CTA 文字大小
//        mediaView.setVolumeImageSize(24) //設定影音廣告靜音圖片大小
//        mediaView.setVideoCountdownTextSize(17) //設定影音倒數文字大小
        nativeAdView.addSubview(mediaView)
        nativeAdView.mediaView = mediaView
        
        let mapNativeAd = TADMediatedUnifiedNativeAd(tadNativeAd: ad, tadNativeAdView: nativeAdView)
        delegate?.customEventNativeAd(self, didReceive: mapNativeAd)
    }
    
    func nativeAd(_ ad: TADNativeAd!, didFailToReceiveAdWithError error: TADRequestError!) {
        delegate?.customEventNativeAd(self, didFailToLoadWithError: error)
    }
}
