//
//  TADMediatedUnifiedNativeAd.swift
//  AdmobSDKTest
//
//  Created by Charles Wang on 2022/1/17.
//

import Foundation
import GoogleMobileAds
import TAMediaAdsFramework

class TADMediatedUnifiedNativeAd: NSObject {
    var tadNativeAd: TADNativeAd
    var tadNativeAdView: TADNativeAdView
    var mappedImages = [GADNativeAdImage]()
    var mapIcon: GADNativeAdImage?
    
    init(tadNativeAd: TADNativeAd, tadNativeAdView: TADNativeAdView) {
        self.tadNativeAdView = tadNativeAdView
        self.tadNativeAd = tadNativeAd
        super.init()
        self.tadNativeAd.delegate = self
        self.tadNativeAdView.mediaView?.statusDelegate = self
        guard let mediaContent = tadNativeAd.adContent.mediaContent else {
            return
        }
        if let tadImage = mediaContent.image1200x627?.image {
            let adImage = GADNativeAdImage(image: tadImage)
            tadNativeAdView.mediaView?.setMainImage(image: tadImage)
            mappedImages.append(adImage)
        }
        if let iconImage = tadNativeAd.adContent.iconSquare?.image {
            mapIcon = GADNativeAdImage(image: iconImage)
        }
    }
    
    deinit {
        print("TADMediatedUnifiedNativeAd - TADMediatedUnifiedNativeAd deinit")
    }
}

extension TADMediatedUnifiedNativeAd: TADNativeAdDelegate {
    func nativeAdDidImpression(_ ad: TADNativeAd!) {
        print("TADMediatedUnifiedNativeAd - nativeAdDidImpression")
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidRecordImpression(self)
    }
    
    func nativeAdDidClick(_ ad: TADNativeAd!) {
        print("TADMediatedUnifiedNativeAd - nativeAdDidClick")
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidRecordClick(self)
    }
}

extension TADMediatedUnifiedNativeAd: TADVideoStatusDelegate {
    func didStartVideo() {
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidPlayVideo(self)
    }
    
    func didPlayVideo() {
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidPlayVideo(self)
    }
    
    func didPauseVideo() {
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidPauseVideo(self)
    }
    
    func didEndVideoPlayback() {
        GADMediatedUnifiedNativeAdNotificationSource.mediatedNativeAdDidEndVideoPlayback(self)
    }
    
    func didMuteVideo() {}
    
    func didUnmuteVideo() {}
}

extension TADMediatedUnifiedNativeAd: GADMediatedUnifiedNativeAd {
    var headline: String? {
        return tadNativeAd.adContent.longSubject
    }
    
    var images: [GADNativeAdImage]? {
        return mappedImages
    }
    
    var body: String? {
        return tadNativeAd.adContent.body
    }
    
    var icon: GADNativeAdImage? {
        return mapIcon
    }
    
    var callToAction: String? {
        return tadNativeAd.adContent.callToAction
    }
    
    var starRating: NSDecimalNumber? {
        return nil
    }
    
    var store: String? {
        return nil
    }
    
    var price: String? {
        return nil
    }
    
    var advertiser: String? {
        return nil
    }
    
    var extraAssets: [String: Any]? {
        return nil
    }
    
    var mediaView: UIView? {
        return tadNativeAdView.mediaView
    }
    
    var hasVideoContent: Bool {
        guard let mediaContent = tadNativeAd.adContent.mediaContent else {
            return false
        }
        return mediaContent.isVideoContent
    }
    
    func didRender(in view: UIView, clickableAssetViews: [GADNativeAssetIdentifier: UIView], nonclickableAssetViews: [GADNativeAssetIdentifier: UIView], viewController: UIViewController) {
        view.subviews.first { $0 is TADNativeAdView }?.removeFromSuperview()
        view.insertSubview(tadNativeAdView, at: 0)
        fillSuperView(view: tadNativeAdView)
        
        if let titleLabel = clickableAssetViews[GADNativeAssetIdentifier.headlineAsset] as? UILabel {
            titleLabel.isUserInteractionEnabled = true
            tadNativeAdView.longSubjectLabel = titleLabel
        }

        if let bodyLabel = clickableAssetViews[GADNativeAssetIdentifier.bodyAsset] as? UILabel {
            bodyLabel.isUserInteractionEnabled = true
            tadNativeAdView.bodyLabel = bodyLabel
        }

        if let ctaLabel = clickableAssetViews[GADNativeAssetIdentifier.callToActionAsset] {
            ctaLabel.isUserInteractionEnabled = true
            tadNativeAdView.callToActionView = ctaLabel
        }
        
        if let icon = clickableAssetViews[GADNativeAssetIdentifier.iconAsset] as? UIImageView {
            icon.isUserInteractionEnabled = true
            tadNativeAdView.squareImgeView = icon
        }

        if let mediaView = clickableAssetViews[GADNativeAssetIdentifier.mediaViewAsset], let tadMediaView = tadNativeAdView.mediaView {
            mediaView.subviews.first { $0 is TADNativeMediaView }?.removeFromSuperview()
            mediaView.addSubview(tadMediaView)
            fillSuperView(view: tadMediaView)
        }
        
        tadNativeAdView.nativeAd = tadNativeAd
    }
    
    func didUntrackView(_ view: UIView?) {
        print("TADMediatedUnifiedNativeAd - didUntrackView")
        tadNativeAdView.removeFromSuperview()
        tadNativeAdView.mediaView?.removeFromSuperview()
    }
}

extension TADMediatedUnifiedNativeAd {
    func fillSuperView(view: UIView) {
        view.frame = view.superview?.bounds ?? .zero
    }
}
